from flask import Blueprint, request, jsonify, make_response, json
from app.devices.models import Device
from app.zone.models import Zone
from app.rooms.models import Room
from app.users.models import User
from app.patients.models import Patient
from app.devices.schemas import device_schema
from sqlalchemy.orm import aliased
from app import db
from app.utils import token_required, delete_record, paginated_query, single_record_query
from app.doctor.models import Doctor
from app.equipment.models import Equipment

devices = Blueprint('devices', __name__)

@devices.route('/api/v1/add/device', methods=['POST'])
@token_required(1, 2, 3)
def create_device(current_user):
    try:
        device_dict = json.loads(request.form.get('formData'))
        db.session.add(Device(**device_schema.load({
            'device_name': device_dict['device_name'].strip(),
            'device_mac_address': device_dict['device_mac_address'].strip(),
            'x_coordinate': device_dict['x_coordinate'] if device_dict['x_coordinate'] else 0,
            'y_coordinate': device_dict['y_coordinate'] if device_dict['y_coordinate'] else 0,
            'zone_id': 1,
            'room_id': 1,
            'site_id': request.args.get('site_id'),
            'user_id': current_user.id,
            'device_type_id': device_dict['device_type_id'],
            'device_type_name': device_dict['device_type_name'],
        })))
        db.session.commit()     
        return make_response(jsonify({'message': 'Device added successfully', "status": "Success"}), 200)
    except Exception as e:
        return make_response(jsonify({'args': e.args, "message": 'Failed to add device', "status": "error"}), 500)
    
@devices.route('/api/v1/get/device_by_id/<int:device_id>', methods=['GET'])
@token_required(1, 2, 3)
def get_device(current_user, device_id):
    try:
        device = single_record_query(Device, Device.device_id, device_id, current_user)
        if device:
            zone = Zone.query.filter_by(zone_id=device.zone_id).first()
            room = Room.query.filter_by(id=device.room_id).first()
            return make_response(jsonify({'device': {
                'id': device.device_id,
                'zone_name': zone.zone_name if zone else "",
                'room_name': room.room_name if room else "",
                'device_name': device.device_name,
                'device_mac_address': device.device_mac_address,
                'x_coordinate': device.x_coordinate,
                'y_coordinate': device.y_coordinate,
                'device_type_name': device.device_type_name,
                'device_type_id': device.device_type_id,
            }}), 200)
        return make_response(jsonify({'message': 'Device not found', "status": "error"}), 404)
    except Exception as e:
        return make_response(jsonify({'message': f'error: {e}', "status": "error"}), 500)
    
@devices.route('/api/v1/get/devices', methods=['GET'])
@token_required(1, 2, 3)
def get_asset_trackers(current_user):
    try:
        devices_query = Device.query.order_by(Device.device_name.asc())
        devices_paginated, data = paginated_query(devices_query,
            [Device.device_mac_address, Device.device_name],
                        Device.site_id)
        rooms = Room.query.filter(Room.id.in_(
            [device.room_id for device in devices_paginated if device.room_id]
        )).all()
        room_map = {room.id: room for room in rooms}
        zone_ids = {room.zone_id for room in rooms if room.zone_id}
        zones = Zone.query.filter(Zone.zone_id.in_(zone_ids)).all()
        zone_map = {zone.zone_id: zone.zone_name for zone in zones}
        serialized_devices = []
        for device in devices_paginated:
            id = device.device_id
            assigned_entity = None
            assigned_entity_name = ""
            assigned_entity_type = ""

            # Check if device is assigned to a patient
            patient = Patient.query.filter_by(device_id=id).first()
            if patient:
                assigned_entity = patient
                assigned_entity_name = patient.patient_name
                assigned_entity_type = "Patient"
            else:
                # Check if device is assigned to a doctor
                doctor = Doctor.query.filter_by(device_id=id).first()
                if doctor:
                    assigned_entity = doctor
                    assigned_entity_name = doctor.doctor_name
                    assigned_entity_type = "Doctor"
                else:
                    # Check if device is assigned to equipment
                    equipment = Equipment.query.filter_by(device_id=id).first()
                    if equipment:
                        assigned_entity = equipment
                        assigned_entity_name = equipment.equipment_name
                        assigned_entity_type = "Equipment"
                    else:
                        # Check if device is assigned to staff
                        user = User.query.filter_by(device_id=id).first()
                        if user and user.full_name:
                            assigned_entity = user
                            user_name = json.loads(user.full_name)
                            assigned_entity_name = f"{user_name.get('first_name', '')} {user_name.get('last_name', '')}"
                            assigned_entity_type = "Staff"

            room = room_map.get(device.room_id)
            zone_name = zone_map.get(room.zone_id, "") if room else ""
            serialized_devices.append({
                'id': id,
                'zone_name': zone_name,
                'room_name': room.room_name if room else "",
                'device_name': device.device_name,
                'device_mac_address': device.device_mac_address,
                'device_type_id': device.device_type_id,
                'assigned_entity_name': assigned_entity_name,
                'assigned_entity_type': assigned_entity_type,
                'x_coordinate': device.x_coordinate,
                'y_coordinate': device.y_coordinate
            })
        if serialized_devices:
            return make_response(jsonify({'device': serialized_devices, "data": data}), 200)
        else:
            return make_response(jsonify({'message': 'No Asset Tracker found for the given site_id',"data": data}), 200)
    except Exception as e:
        return make_response(jsonify({'message': f'Error: {e}', "status": "error"}), 500)

@devices.route('/api/v1/update/device/<int:device_id>', methods=['PUT'])
@token_required(1, 2, 3)
def update_asset_tracker(current_user, device_id):
    try:
        device = single_record_query(Device, Device.device_id, device_id, current_user)
        if not device:
            return make_response(jsonify({"message": "Asset tracker device not found", "status": "error"}), 404)
        device_dict = json.loads(request.form.get('formData'))
        for key, value in device_schema.load({
            'device_name': device_dict.get('device_name').strip(),
            'device_mac_address': device_dict.get('device_mac_address').strip(),
            'x_coordinate': device_dict.get('x_coordinate'),
            'y_coordinate': device_dict.get('y_coordinate'),
            'zone_id': device_dict.get('zone_id', device.zone_id),
            'room_id': device_dict.get('room_id', device.room_id),
            'device_type_id': device_dict.get('device_type_id', device.device_type_id),
            'device_type_name': device_dict.get('device_type_name'),
        }).items():
            setattr(device, key, value)
        db.session.commit()
        return make_response(jsonify({'message': 'Device updated successfully', "status": "Success"}), 200)
    except Exception as e:
        return make_response(jsonify({'args': e.args, "message": 'Failed to update asset tracker device', "status": "error"}), 500)

@devices.route('/api/v1/delete/device/<id>', methods=['DELETE'])
@token_required(1, 2, 3)
def delete_device(current_user, id):
    return delete_record(Device, current_user, id, "Device", "device_id")

@devices.route('/lock-data', methods=['GET'])
def get_lock_data():
    with open('/logs/lock_data.json', 'r') as lock_file:
        lock_data = json.load(lock_file)
        data = str(lock_data)
    return jsonify(data)

@devices.route('/devices-search-list', methods=['GET'])
@token_required(1, 2, 3)
def device_name(current_user):
    try:
        search = request.args.get('search')
        # Get all device IDs that are already assigned to any entity
        assigned_device_ids = db.session.query(Patient.device_id).union(
            db.session.query(Doctor.device_id)
        ).union(
            db.session.query(Equipment.device_id)
        ).union(
            db.session.query(User.device_id)
        ).all()
        assigned_device_ids = [device_id[0] for device_id in assigned_device_ids]

        # Query devices that are not assigned to any entity
        device_query = Device.query.filter_by(site_id=request.args.get('site_id'))
        if assigned_device_ids:
            device_query = device_query.filter(~Device.device_id.in_(assigned_device_ids))
        if search:
            device_query = device_query.filter(Device.device_name.ilike(f'%{search}%'))

        filtered_devices = device_query.all()
        if not filtered_devices:
            serialized_devices = [
                {
                    "device_id": "",
                    "device_name": "Device not available",
                    "room_id": "",
                    "room_name": ""
                }
            ]
        else:
            serialized_devices = [
                {
                    "device_id": device.device_id,
                    "device_name": device.device_name,
                    "room_id": device.room_id,
                    "room_name": Room.query.filter_by(id=device.room_id).first().room_name if device.room_id else ""
                }
                for device in filtered_devices
            ]
        return jsonify({"device": serialized_devices}), 200
    except Exception as e:
        return make_response(jsonify({"Error": str(e)}), 500)